==> unicore-uvos-server-1.5.1-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/server/idresolver/Authenticator.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 17, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.server.idresolver;

import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.service.invoker.AbstractInvoker;

import eu.unicore.security.SecurityTokens;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.AuthenticationException;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.engine.InternalAuthorization;
import pl.edu.icm.unicore.uvos.engine.SecurityContext;
import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;

/**
 * Utility methods to map credentials coming from client (anyhow, e.g. as SSL cert)
 * on {@link Identity} and authenticating it. The result of the first successful 
 * authentication is returned as {@link SecurityContext} instance. 
 * The order of the different kinds of mappings is configurable on identity type and 
 * data source base. 
 * 
 * @author K. Benedyczak
 */
public class Authenticator
{
	private static ThreadLocal<SecurityContext> authnContext = new ThreadLocal<SecurityContext>();
	
	private static final Logger log = Logger.getLogger(Authenticator.class);
	
	private List<IResolver> resolvers = new ArrayList<IResolver>();
	protected InternalAuthorization voAuthz;
	private boolean failOnInvalid;
	private boolean tlsDNFirst;
	
	public Authenticator(Configuration conf, InternalAuthorization voAuthz)
	{
		this.voAuthz = voAuthz;
		failOnInvalid = conf.getAsBool(ConfDefaults.AUTHN_FAIL, 
				ConfDefaults.authnFail, log);
		
		String rawOrder = conf.getAsString(ConfDefaults.AUTHN_ORDER, 
				ConfDefaults.authnOrder, log);
		parseOrder(rawOrder);
		if (resolvers.size() == 0)
		{
			log.error("No valid authN possibility found. Will use default: " +
					ConfDefaults.authnOrder);
			parseOrder(ConfDefaults.authnOrder);
		}
		
		tlsDNFirst = conf.getAsBool(ConfDefaults.AUTHN_TLS_DN_FIRST, 
			ConfDefaults.authnTlsDNFirst, log);
	}

	
	/**
	 * Tries to authenticate. Wraps resolve() adding logging of internal errors exceptions
	 * and converts an error to generic AuthenticationException, which will be sent to the client.
	 */
	public void authenticate() 
		throws AuthenticationException
	{
		try
		{
			resolve();
		} catch (IdResolveException e)
		{
			log.error(e.toString());
			throw new AuthenticationException("Internal error in authN stack: " + e);
		} catch (IdUnresolvedException e)
		{
			throw new AuthenticationException(e.getMessage());
		} catch (UVOSException e)
		{
			log.error("General authentication problem", e);
			throw new AuthenticationException(e.toString());
		}
	}

	public static X509Certificate getAuthNCert()
	{
		MessageContext ctx = AbstractInvoker.getContext();
		SecurityTokens tokens = (SecurityTokens) 
			ctx.getProperty(SecurityTokens.KEY);
		if (tokens != null)
			return tokens.getEffectiveUserCertificate();
		return null;
	}
	
	
	private void parseOrder(String rawOrder)
	{
		String [] orderS = rawOrder.split(" ");
		for (String authSrc: orderS)
		{
			if (authSrc.equals("TLS"))
				resolvers.add(new TLSResolver(tlsDNFirst, voAuthz));
			else if (authSrc.equals("HTTP"))
				resolvers.add(new LoginResolver(voAuthz));
			else
				log.warn("Unknown authN possibility '" + 
						authSrc + "' - IGNORED");
		}
	}
	
	public void resolve() throws IdResolveException, 
		IdUnresolvedException, UVOSException
	{
		MessageContext ctx = AbstractInvoker.getContext();
		SecurityTokens tokens = (SecurityTokens) 
			ctx.getProperty(SecurityTokens.KEY);
		if (tokens == null)
			throw new IdResolveException("Client not authenticated, " +
					"error in security handlers configuration");
		
		for (IResolver resolver: resolvers)
		{
			SecurityContext caller;
			try
			{
				caller = resolver.resolve(tokens);
			} catch (IdUnresolvedException e)
			{
				if (failOnInvalid)
					throw e;
				continue;
			}
			//caller == null simply means that there was no input for the resolver 
			//so continue silently.
			if (caller != null)
			{
				authnContext.set(caller);
				return;
			}
		}			
		throw new IdUnresolvedException("Client not authenticated");
	}
	
	public static SecurityContext getCurrentContext() throws InternalException
	{
		SecurityContext ctx = authnContext.get();
		if (ctx == null)
		{
			log.fatal("SecurityContext is not set, so authentication was not performed.");
			Thread.dumpStack();
			throw new InternalException("Authentication problem.");
		}
		return ctx;
	}
	
	public static void injectFakeContext(SecurityContext ctx)
	{
		authnContext.set(ctx);
	}
}

==> unicore-uvos-server-1.6.0-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/server/idresolver/Authenticator.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 17, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.server.idresolver;

import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.service.invoker.AbstractInvoker;

import eu.unicore.security.SecurityTokens;
import eu.unicore.util.Log;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.AuthenticationException;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.engine.InternalAuthorization;
import pl.edu.icm.unicore.uvos.engine.SecurityContext;
import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;

/**
 * Utility methods to map credentials coming from client (anyhow, e.g. as SSL cert)
 * on {@link Identity} and authenticating it. The result of the first successful 
 * authentication is returned as {@link SecurityContext} instance. 
 * The order of the different kinds of mappings is configurable on identity type and 
 * data source base. 
 * 
 * @author K. Benedyczak
 */
public class Authenticator
{
	private static ThreadLocal<SecurityContext> authnContext = new ThreadLocal<SecurityContext>();
	
	private static final Logger log = Log.getLogger(UVOSServerProperties.LOG_PFX, Authenticator.class);
	
	private List<IResolver> resolvers = new ArrayList<IResolver>();
	protected InternalAuthorization voAuthz;
	private boolean failOnInvalid;
	private boolean tlsDNFirst;
	
	public Authenticator(UVOSServerProperties conf, InternalAuthorization voAuthz)
	{
		this.voAuthz = voAuthz;
		failOnInvalid = conf.getBooleanValue(UVOSServerProperties.AUTHN_FAIL);
		
		String rawOrder = conf.getValue(UVOSServerProperties.AUTHN_ORDER);
		parseOrder(rawOrder);
		if (resolvers.size() == 0)
		{
			log.error("No valid authN possibility found. Will use default: " +
					UVOSServerProperties.defaults.get(UVOSServerProperties.AUTHN_ORDER));
			parseOrder(UVOSServerProperties.defaults.get(UVOSServerProperties.AUTHN_ORDER).getDefault());
		}
		
		tlsDNFirst = conf.getBooleanValue(UVOSServerProperties.AUTHN_TLS_DN_FIRST);
	}

	
	/**
	 * Tries to authenticate. Wraps resolve() adding logging of internal errors exceptions
	 * and converts an error to generic AuthenticationException, which will be sent to the client.
	 */
	public void authenticate() 
		throws AuthenticationException
	{
		try
		{
			resolve();
		} catch (IdResolveException e)
		{
			log.error(e.toString());
			throw new AuthenticationException("Internal error in authN stack: " + e);
		} catch (IdUnresolvedException e)
		{
			throw new AuthenticationException(e.getMessage());
		} catch (UVOSException e)
		{
			log.error("General authentication problem", e);
			throw new AuthenticationException(e.toString());
		}
	}

	public static X509Certificate getAuthNCert()
	{
		MessageContext ctx = AbstractInvoker.getContext();
		SecurityTokens tokens = (SecurityTokens) 
			ctx.getProperty(SecurityTokens.KEY);
		if (tokens != null)
			return tokens.getEffectiveUserCertificate();
		return null;
	}
	
	
	private void parseOrder(String rawOrder)
	{
		String [] orderS = rawOrder.split(" ");
		for (String authSrc: orderS)
		{
			if (authSrc.equals("TLS"))
				resolvers.add(new TLSResolver(tlsDNFirst, voAuthz));
			else if (authSrc.equals("HTTP"))
				resolvers.add(new LoginResolver(voAuthz));
			else
				log.warn("Unknown authN possibility '" + 
						authSrc + "' - IGNORED");
		}
	}
	
	public void resolve() throws IdResolveException, 
		IdUnresolvedException, UVOSException
	{
		MessageContext ctx = AbstractInvoker.getContext();
		SecurityTokens tokens = (SecurityTokens) 
			ctx.getProperty(SecurityTokens.KEY);
		if (tokens == null)
			throw new IdResolveException("Client not authenticated, " +
					"error in security handlers configuration");
		
		for (IResolver resolver: resolvers)
		{
			SecurityContext caller;
			try
			{
				caller = resolver.resolve(tokens);
			} catch (IdUnresolvedException e)
			{
				if (failOnInvalid)
					throw e;
				continue;
			}
			//caller == null simply means that there was no input for the resolver 
			//so continue silently.
			if (caller != null)
			{
				authnContext.set(caller);
				return;
			}
		}			
		throw new IdUnresolvedException("Client not authenticated");
	}
	
	public static SecurityContext getCurrentContext() throws InternalException
	{
		SecurityContext ctx = authnContext.get();
		if (ctx == null)
		{
			log.fatal("SecurityContext is not set, so authentication was not performed.");
			Thread.dumpStack();
			throw new InternalException("Authentication problem.");
		}
		return ctx;
	}
	
	public static void injectFakeContext(SecurityContext ctx)
	{
		authnContext.set(ctx);
	}
}
